@@ -75,7 +75,7 @@ static void
 gdiplus_init(void)
 {
   GpStatus s;
-  static GdiplusStartupInput gi = (GdiplusStartupInput){1, NULL, FALSE, FALSE};
+  static GdiplusStartupInput gi = {1, NULL, FALSE, FALSE};
   static ULONG_PTR gis = 0;
   if (!gis) {
     s = GdiplusStartup(&gis, &gi, NULL);
@@ -559,6 +559,8 @@ draw_img(HDC dc, imglist * img)
     int height = img->pixelheight;
     left += PADDING;
     top += OFFSET + PADDING;
+    // adjust horizontal scrolling
+    left -= horclip();
 
     int coord_transformed = 0;
     XFORM old_xform;
@@ -699,6 +701,8 @@ winimgs_paint(void)
 #endif
     } else {
       int left = img->left;
+      // do not adjust horizontal scrolling here
+
       int top = img->top - term.virtuallines - term.disptop;
       if (top + img->height < 0 || top > term.rows) {
         // if the image is scrolled out, serialize it into a temp file
@@ -779,6 +783,11 @@ winimgs_paint(void)
         int ybot = min(top + img->height, term.rows) * cell_height + OFFSET + PADDING;
         int xlft = left * cell_width + PADDING;
         int xrgt = min(left + img->width, term.cols) * cell_width + PADDING;
+
+        // adjust horizontal scrolling
+        xlft -= horclip();
+        xrgt -= horclip();
+
         if (img->len) {
           // better background handling implemented below; this version 
           // would expose artefacts if a transparent image is scrolled
@@ -850,6 +859,9 @@ winimgs_paint(void)
             ExcludeClipRect(dc, xlft, ytop, xrgt, ybot);
           }
           else {
+            // adjust horizontal scrolling
+            left -= horclip() / cell_width;
+
             StretchBlt(dc,
                        left * cell_width + PADDING, top * cell_height + OFFSET + PADDING,
                        img->width * cell_width, img->height * cell_height,
@@ -909,7 +921,7 @@ winimgs_paint(void)
 #include "charset.h"  // path_win_w_to_posix
 
 void
-win_emoji_show(int x, int y, wchar * efn, void * * bufpoi, int * buflen, int elen, ushort lattr, bool italic)
+win_emoji_show(int x0, int y, wchar * efn, void * * bufpoi, int * buflen, int elen, ushort lattr, bool italic)
 {
   gdiplus_init();
 
@@ -961,7 +973,7 @@ win_emoji_show(int x, int y, wchar * efn, void * * bufpoi, int * buflen, int ele
     gpcheck("load file", s);
   }
 
-  int col = PADDING + x * cell_width;
+  int col = PADDING + x0 * cell_width - horclip();
   int row = OFFSET + PADDING + y * cell_height;
   if ((lattr & LATTR_MODE) >= LATTR_BOT)
     row -= cell_height;
@@ -969,7 +981,7 @@ win_emoji_show(int x, int y, wchar * efn, void * * bufpoi, int * buflen, int ele
   if ((lattr & LATTR_MODE) != LATTR_NORM) {
     w *= 2;
     // fix position in double-width line
-    col += x * cell_width;
+    col += x0 * cell_width;
   }
   int h = cell_height;
   if ((lattr & LATTR_MODE) >= LATTR_TOP)
@@ -1082,6 +1094,12 @@ save_img(HDC dc, int x, int y, int w, int h, wstring fn)
   DeleteDC(copdc);
   DeleteObject(copbm);
 
+  // create the image file so we get proper access rights
+  char * fnp = path_win_w_to_posix(fn);
+  int f = open(fnp, O_WRONLY | O_CREAT | O_EXCL, 0600);
+  close(f);
+  free(fnp);
+
   // save as png
   CLSID png;
   CLSIDFromString(W("{557CF406-1A04-11D3-9A73-0000F81EF32E}"), &png);
